package com.qmth.paper.library.business.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.paper.library.business.bean.result.BindDetailResult;
import com.qmth.paper.library.business.bean.result.PaperScanTaskDetailResult;
import com.qmth.paper.library.business.bean.result.PaperScanTaskResult;
import com.qmth.paper.library.business.entity.PaperScanTask;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Set;

/**
 * <p>
 * 扫描任务表 Mapper 接口
 * </p>
 */
public interface PaperScanTaskMapper extends BaseMapper<PaperScanTask> {

    /**
     * 扫描任务分页查询
     *
     * @param iPage           分页参数
     * @param schoolId        学校id
     * @param courseName      课程名称
     * @param teacherName     教师名称
     * @param smallerProgress 进度阈值（小）
     * @param largerProgress  进度阈值（大）
     * @param orgIds          数据权限
     * @return 分页查询结果
     */
    IPage<PaperScanTaskResult> findPaperScanTaskPage(@Param("iPage") Page<PaperScanTaskResult> iPage,
                                                     @Param("schoolId") Long schoolId,
                                                     @Param("courseName") String courseName,
                                                     @Param("teacherName") String teacherName,
                                                     @Param("smallerProgress") int smallerProgress,
                                                     @Param("largerProgress") int largerProgress,
                                                     @Param("orgIds") Set<Long> orgIds);

    /**
     * 根据扫描任务id分页查询扫描任务详情
     *
     * @param iPage           分页参数
     * @param schoolId        学校id
     * @param paperScanTaskId 扫描任务id
     * @param orgIds          数据权限
     * @return 分页查询结果
     */
    IPage<PaperScanTaskDetailResult> findPaperScanTaskDetailPageByScanTaskId(@Param("iPage") Page<PaperScanTaskDetailResult> iPage,
                                                                             @Param("schoolId") Long schoolId,
                                                                             @Param("paperScanTaskId") Long paperScanTaskId,
                                                                             @Param("orgIds") Set<Long> orgIds);

    /**
     * 根据扫描任务id查询扫描任务的绑定详情
     *
     * @param schoolId        学校id
     * @param paperScanTaskId 扫描任务id
     * @return 查询结果
     */
    List<BindDetailResult> findBindDetailByScanTaskId(@Param("schoolId") Long schoolId,
                                                      @Param("paperScanTaskId") Long paperScanTaskId);
}
