package com.qmth.paper.library.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.paper.library.business.bean.result.PaperArchivesResult;
import com.qmth.paper.library.business.bean.result.SelectResult;
import com.qmth.paper.library.business.entity.PaperArchives;
import com.qmth.paper.library.business.entity.PaperScanTask;
import com.qmth.paper.library.business.mapper.PaperArchivesMapper;
import com.qmth.paper.library.business.service.PaperArchivesService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.util.ServletUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 档案管理表 服务实现类
 * </p>
 */
@Service
public class PaperArchivesServiceImpl extends ServiceImpl<PaperArchivesMapper, PaperArchives> implements PaperArchivesService {

    @Resource
    PaperScanTaskService paperScanTaskService;

    @Override
    public IPage<PaperArchivesResult> pageData(Long semesterId, Long managerOrgId, Long archivesTypeId, String archivesName, Integer pageNumber, Integer pageSize) {
        return this.baseMapper.pageData(new Page<>(pageNumber, pageSize), semesterId, managerOrgId, archivesTypeId, archivesName);
    }

    @Override
    public boolean saveData(PaperArchives paperArchives) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        PaperArchives archives = this.baseMapper.selectByArchivesName(schoolId, paperArchives.getArchivesName());
        if (archives != null) {
            throw ExceptionResultEnum.ERROR.exception("档案名称已存在");
        }
        paperArchives.setId(SystemConstant.getDbUuid());
        paperArchives.setSchoolId(schoolId);
        // 档案编号（时间戳）
        paperArchives.setArchivesCode(String.valueOf(System.currentTimeMillis()));
        paperArchives.setCreateId(sysUser.getId());
        paperArchives.setCreateTime(System.currentTimeMillis());
        return this.save(paperArchives);
    }

    @Override
    public boolean delete(Long id) {
        PaperArchives paperArchives = this.getById(id);
        if (paperArchives == null) {
            throw ExceptionResultEnum.ERROR.exception("数据有误");
        }

        List<PaperScanTask> paperScanTaskList = paperScanTaskService.listByPaperArchivesId(id);
        if (!paperScanTaskList.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("已有扫描任务，不能删除");
        }
        return this.removeById(id);
    }

    @Override
    public List<SelectResult> queryManagerOrgs(Long semesterId) {
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        return this.baseMapper.queryManagerOrgs(schoolId, semesterId);
    }

}
