package com.qmth.paper.library.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.paper.library.business.bean.result.SelectResult;
import com.qmth.paper.library.business.entity.PaperArchivesType;
import com.qmth.paper.library.business.mapper.PaperArchivesTypeMapper;
import com.qmth.paper.library.business.service.PaperArchivesTypeService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.util.ServletUtil;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 档案类型表 服务实现类
 * </p>
 */
@Service
public class PaperArchivesTypeServiceImpl extends ServiceImpl<PaperArchivesTypeMapper, PaperArchivesType> implements PaperArchivesTypeService {

    @Override
    public IPage<PaperArchivesType> pageData(Integer pageNumber, Integer pageSize) {
        QueryWrapper<PaperArchivesType> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().orderByAsc(PaperArchivesType::getCreateTime);
        return this.baseMapper.selectPage(new Page<>(pageNumber, pageSize), queryWrapper);
    }

    @Override
    public boolean saveData(PaperArchivesType paperArchivesType) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        PaperArchivesType archivesType = this.baseMapper.selectByArchivesType(schoolId, paperArchivesType.getName());

        // 新增
        if (paperArchivesType.getId() == null) {
            if (archivesType != null) {
                throw ExceptionResultEnum.ERROR.exception("类型名称已存在");
            }
            paperArchivesType.setId(SystemConstant.getDbUuid());
            paperArchivesType.setSchoolId(schoolId);
            paperArchivesType.setCreateId(sysUser.getCreateId());
            paperArchivesType.setCreateTime(System.currentTimeMillis());
            return this.save(paperArchivesType);
        }
        // 修改
        else {
            if (archivesType == null) {
                throw ExceptionResultEnum.ERROR.exception("数据有误");
            } else if (!paperArchivesType.getId().equals(archivesType.getId())) {
                throw ExceptionResultEnum.ERROR.exception("类型名称已存在");
            }
            archivesType.setName(paperArchivesType.getName());
            archivesType.setUpdateId(sysUser.getId());
            archivesType.setUpdateTime(System.currentTimeMillis());
            return this.updateById(archivesType);
        }
    }

    @Override
    public boolean delete(Long id) {
        PaperArchivesType paperArchivesType = this.getById(id);
        if (paperArchivesType == null) {
            throw ExceptionResultEnum.ERROR.exception("数据有误");
        }
        return this.removeById(id);
    }

    @Override
    public List<SelectResult> queryData() {
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        return this.baseMapper.queryData(schoolId);
    }
}
