package com.qmth.paper.library.common.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.paper.library.common.bean.dto.TaskPrintClassDto;
import com.qmth.paper.library.common.bean.result.BasicStudentResult;
import com.qmth.paper.library.common.entity.BasicStudent;
import org.apache.ibatis.annotations.Param;

import java.util.List;

import java.util.Set;

/**
 * <p>
 * 学校id Mapper 接口
 * </p>
 *
 * @author wangliang
 * @since 2021-08-02
 */
public interface BasicStudentMapper extends BaseMapper<BasicStudent> {
    /**
     * 分页查询学生信息
     *
     * @param iPage       分页参数
     * @param queryParams 查询条件
     * @param schoolId    学校id
     * @param collegeId   学院id
     * @param majorName   专业名称
     * @param clazzName   班级名称
     * @param orgIds      数据权限
     * @return 结果
     */
    IPage<BasicStudentResult> findBasicStudentPage(@Param("iPage") Page<BasicStudentResult> iPage,
                                                   @Param("queryParams") String queryParams,
                                                   @Param("schoolId") Long schoolId,
                                                   @Param("collegeId") Long collegeId,
                                                   @Param("majorName") String majorName,
                                                   @Param("clazzName") String clazzName,
                                                   @Param("orgIds") Set<Long> orgIds);

    /**
     * 根据id查找学生信息
     *
     * @param id id
     * @return 结果
     */
    BasicStudentResult findById(@Param("id") Long id);

    /**
     * 根据第三方字典id查找学生集合
     *
     * @param schoolId        学校id
     * @param collegeId       学生学院id
     * @param majorName       专业id
     * @param clazzName       班级id
     * @param studentCodeList 学号集合
     * @return 学生查询结果
     */
    List<BasicStudentResult> findBasicStudentList(@Param("schoolId") Long schoolId,
                                                  @Param("collegeId") Long collegeId,
                                                  @Param("majorName") String majorName,
                                                  @Param("clazzName") String clazzName,
                                                  @Param("studentCodeList") List<String> studentCodeList);
}
