package com.qmth.paper.library.common.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.paper.library.common.bean.dto.TaskPrintClassDto;
import com.qmth.paper.library.common.bean.params.BasicStudentExtrasParam;
import com.qmth.paper.library.common.bean.params.BasicStudentParams;
import com.qmth.paper.library.common.bean.result.BasicStudentResult;
import com.qmth.paper.library.common.entity.BasicStudent;
import com.qmth.paper.library.common.entity.SysUser;
import org.springframework.util.LinkedMultiValueMap;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 学校id 服务类
 * </p>
 *
 * @author wangliang
 * @since 2021-08-02
 */
public interface BasicStudentService extends IService<BasicStudent> {
    /**
     * 学生基础信息分页查询
     *
     * @param queryParams 查询参数（学生学号或姓名）
     * @param collegeId   学院id
     * @param clazzName   专业名称
     * @param majorName   班级名称
     * @param pageNumber  分页页码
     * @param pageSize    分页容量
     * @return 分页结果
     */
    IPage<BasicStudentResult> basicStudentPage(String queryParams, Long collegeId, String clazzName, String majorName, int pageNumber, int pageSize);

    /**
     * 根据id查询基础学生数据
     *
     * @param id 学生id
     * @return 学生信息
     */
    BasicStudentResult findById(Long id);

    /**
     * 学生基础信息查询
     *
     * @param schoolId        学校id
     * @param collegeId       学院id
     * @param majorName       专业名称
     * @param clazzName       班级名称
     * @param studentCodeList 学号集合
     * @return 学生查询结果
     */
    List<BasicStudentResult> basicStudentList(Long schoolId, Long collegeId, String majorName, String clazzName, List<String> studentCodeList);

    /**
     * 新增/编辑 学生基础信息
     *
     * @param basicStudentParams 基础学生信息参数
     * @param requestUser        新增或编辑的校区id
     * @return id
     */
    Long saveBasicStudent(BasicStudentParams basicStudentParams, SysUser requestUser);

    /**
     * 批量删除学生基本信息（逻辑）
     *
     * @param idList 要删除的学生主键集合
     * @return 是否删除成功
     */
    Boolean removeBasicStudentBatch(List<Long> idList);

    /**
     * 根据条件删除基础学生信息
     *
     * @param queryParams 学生参数（学号或姓名）
     * @param collegeId   学院主键
     * @param majorName   专业主键
     * @param clazzName   班级主键集合
     * @return 是否删除成功
     */
    Boolean removeBasicStudentByCondition(String queryParams, Long collegeId, String majorName, String clazzName);

    /**
     * 执行批量导入学生逻辑
     *
     * @param finalList 基础学生数据集合
     * @param map       参数
     * @return 结果
     */
    Map<String, Object> executeBasicStudentImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) throws IllegalAccessException;
}