package com.qmth.paper.library.api;


import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.paper.library.business.bean.result.EditResult;
import com.qmth.paper.library.business.entity.PaperArchives;
import com.qmth.paper.library.business.service.PaperArchivesService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.util.Result;
import com.qmth.paper.library.common.util.ResultUtil;
import com.qmth.paper.library.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

/**
 * <p>
 * 档案管理表 前端控制器
 * </p>
 */
@Api(tags = "档案管理表controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.paper}/archives")
@Validated
@Aac(strict = BOOL.FALSE, auth = BOOL.FALSE)
public class PaperArchivesController {
    @Resource
    private PaperArchivesService paperArchivesService;

    @ApiOperation(value = "分页查询")
    @PostMapping("/page")
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result page(@ApiParam(value = "档案所属学期") @RequestParam(required = false) Long semesterId,
                       @ApiParam(value = "档案管理部门") @RequestParam(required = false) Long managerOrgId,
                       @ApiParam(value = "档案类型") @RequestParam(required = false) Long archivesTypeId,
                       @ApiParam(value = "档案名称") @RequestParam(required = false) String archivesName,
                       @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(paperArchivesService.pageData(semesterId, managerOrgId, archivesTypeId, archivesName, pageNumber, pageSize));
    }

    @ApiOperation(value = "新增")
    @PostMapping("/save")
    @ApiResponses({@ApiResponse(code = 200, message = "新增成功", response = EditResult.class)})
    public Result save(@Valid @RequestBody PaperArchives paperArchives, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        return ResultUtil.ok(paperArchivesService.saveData(paperArchives));
    }


    @ApiOperation(value = "删除")
    @PostMapping("/delete")
    @ApiResponses({@ApiResponse(code = 200, message = "删除成功", response = Result.class)})
    public Result delete(@ApiParam(value = "主键") @RequestParam(required = false) Long id) {
        return ResultUtil.ok(paperArchivesService.delete(id));
    }

    @ApiOperation(value = "档案管理部门查询")
    @PostMapping("/_manager_orgs/query")
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result queryManagerOrgs(@RequestParam(value = "semesterId", required = false) Long semesterId) {
        return ResultUtil.ok(paperArchivesService.queryManagerOrgs(semesterId));
    }

}
