package com.qmth.paper.library.api;


import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.paper.library.business.bean.result.EditResult;
import com.qmth.paper.library.business.entity.PaperArchivesType;
import com.qmth.paper.library.business.service.PaperArchivesTypeService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.util.Result;
import com.qmth.paper.library.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

/**
 * <p>
 * 档案类型 前端控制器
 * </p>
 */
@Api(tags = "档案类型controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.paper}/archives_type")
@Validated
@Aac(strict = BOOL.FALSE, auth = BOOL.FALSE)
public class PaperArchivesTypeController {
    @Resource
    private PaperArchivesTypeService paperArchivesTypeService;

    @ApiOperation(value = "分页查询")
    @PostMapping("/page")
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result page(@ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(paperArchivesTypeService.pageData(pageNumber, pageSize));
    }

    @ApiOperation(value = "查询-不分页")
    @PostMapping("/query")
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result query() {
        return ResultUtil.ok(paperArchivesTypeService.queryData());
    }

    @ApiOperation(value = "新增")
    @PostMapping("/save")
    @ApiResponses({@ApiResponse(code = 200, message = "保存/更新成功", response = EditResult.class)})
    public Result save(@Valid @RequestBody PaperArchivesType paperArchivesType, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        return ResultUtil.ok(paperArchivesTypeService.saveData(paperArchivesType));
    }


    @ApiOperation(value = "删除")
    @PostMapping("/delete")
    @ApiResponses({@ApiResponse(code = 200, message = "删除成功", response = Result.class)})
    public Result delete(@ApiParam(value = "主键") @RequestParam(required = false) Long id) {
        return ResultUtil.ok(paperArchivesTypeService.delete(id));
    }

}
