package com.qmth.paper.library.api;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.paper.library.business.bean.params.PaperScanTaskParam;
import com.qmth.paper.library.business.bean.result.EditResult;
import com.qmth.paper.library.business.service.PaperLibraryCommonService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.business.templete.execute.AsyncPaperScanTaskImportService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.entity.TBTask;
import com.qmth.paper.library.common.enums.TaskTypeEnum;
import com.qmth.paper.library.common.util.Result;
import com.qmth.paper.library.common.util.ResultUtil;
import com.qmth.paper.library.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description: 试卷扫描任务管理
 * @Author: CaoZixuan
 * @Date: 2022-09-29
 */

@Api(tags = "扫描任务管理controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.paper}/scan_task")
@Validated
@Aac(strict = BOOL.FALSE, auth = BOOL.FALSE)
public class PaperScanTaskController {
    @Resource
    private PaperScanTaskService paperScanTaskService;
    @Resource
    private PaperLibraryCommonService paperLibraryCommonService;
    @Resource
    private AsyncPaperScanTaskImportService asyncPaperScanTaskImportService;

    @ApiOperation(value = "扫描任务管理-查询")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result findPaperScanTaskPage(@ApiParam(value = "课程名称") @RequestParam(required = false) String courseName,
                                        @ApiParam(value = "任课教师") @RequestParam(required = false) String teacherName,
                                        @ApiParam(value = "进度阈值（小）", required = true) @RequestParam @Min(0) Integer smallerProgress,
                                        @ApiParam(value = "进度阈值（大）", required = true) @RequestParam @Min(0) @Max(100) Integer largerProgress,
                                        @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                        @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(paperScanTaskService.paperScanTaskPage(courseName, teacherName, pageNumber, smallerProgress, largerProgress, pageSize, requestUser));
    }

    @ApiOperation(value = "扫描任务管理-新增/编辑")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "更新成功", response = EditResult.class)})
    public Result savePaperScanTask(@Valid @RequestBody PaperScanTaskParam paperScanTaskParam, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(paperScanTaskService.editPaperScanTask(paperScanTaskParam, sysUser));
    }

    @ApiOperation(value = "扫描任务管理-批量删除（物理）")
    @RequestMapping(value = "/delete_batch", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "删除成功", response = EditResult.class)})
    public Result deletePaperScanTask(@ApiParam(value = "选择的要删除的学生id集合", required = true) @RequestParam List<String> idList) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        paperScanTaskService.deletePaperScanTaskByIdList(idList.stream().map(SystemConstant::convertIdToLong).collect(Collectors.toList()), sysUser);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "扫描任务管理-批量导入（异步）")
    @RequestMapping(value = "/scan_task_import", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result paperScanTaskImportAsync(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                                           @ApiParam(value = "档案id", required = true) @RequestParam Long paperArchivesId) throws Exception {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Map<String, Object> map = paperLibraryCommonService.saveTask(file, TaskTypeEnum.PAPER_SCAN_TASK_IMPORT);
        map.put(SystemConstant.SYS_USER, sysUser);
        map.put("paperArchivesId", paperArchivesId);
        asyncPaperScanTaskImportService.importTask(map);
        TBTask tbTask = Objects.nonNull(map.get(SystemConstant.TASK)) ? (TBTask) map.get(SystemConstant.TASK) : null;
        return Objects.nonNull(tbTask) ? ResultUtil.ok(tbTask.getId()) : ResultUtil.error("创建任务失败");
    }

    @ApiOperation(value = "扫描任务管理-绑定扫描员导入模板下载")
    @RequestMapping(value = "/scanner_export", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result scannerExport() throws Exception {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        paperScanTaskService.exportScanner(requestUser);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "扫描任务管理-任务详情查询")
    @RequestMapping(value = "/task_detail_page", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result findPaperScanTaskDetailPage(@ApiParam(value = "扫描任务id", required = true) @RequestParam String paperScanTaskId,
                                              @ApiParam(value = "分页页码", required = true) @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                              @ApiParam(value = "分页数", required = true) @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(paperScanTaskService.findPaperScanTaskDetailPageByScanTaskId(SystemConstant.convertIdToLong(paperScanTaskId), pageNumber, pageSize, requestUser));
    }

    @ApiOperation(value = "扫描任务管理-清除数据")
    @RequestMapping(value = "/clear_scan_data", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "删除成功", response = EditResult.class)})
    public Result clearScanData(@ApiParam(value = "扫描任务id", required = true) @RequestParam String paperScanTaskId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        paperScanTaskService.clearScanData(SystemConstant.convertIdToLong(paperScanTaskId), sysUser);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "扫描任务管理-绑定进度详情")
    @RequestMapping(value = "/bind_progress_detail", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "删除成功", response = EditResult.class)})
    public Result bindProgressDetail(@ApiParam(value = "扫描任务id", required = true) @RequestParam String paperScanTaskId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        return ResultUtil.ok(paperScanTaskService.findBindDetailByScanTaskId(SystemConstant.convertIdToLong(paperScanTaskId), sysUser));
    }

    @ApiOperation(value = "扫描任务管理-释放正在绑定的任务")
    @RequestMapping(value = "/release_binding_task", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "删除成功", response = EditResult.class)})
    public Result releaseBindingTask(@ApiParam(value = "扫描任务id", required = true) @RequestParam String paperScanTaskId,
                                     @ApiParam(value = "数据检察员id", required = true) @RequestParam String bindUserId) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        paperScanTaskService.releaseBindingTask(SystemConstant.convertIdToLong(paperScanTaskId), SystemConstant.convertIdToLong(bindUserId), sysUser);
        return ResultUtil.ok();
    }
}
