package com.qmth.paper.library.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.paper.library.business.bean.params.LoginParam;
import com.qmth.paper.library.business.bean.result.DictionaryResult;
import com.qmth.paper.library.business.bean.result.EditResult;
import com.qmth.paper.library.business.enums.DictionaryEnum;
import com.qmth.paper.library.business.enums.LoginTypeEnum;
import com.qmth.paper.library.business.service.PaperLibraryCommonService;
import com.qmth.paper.library.common.bean.auth.AuthBean;
import com.qmth.paper.library.common.bean.result.LoginResult;
import com.qmth.paper.library.common.bean.result.UserLoginCheckResult;
import com.qmth.paper.library.common.config.DictionaryConfig;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.*;
import com.qmth.paper.library.common.enums.*;
import com.qmth.paper.library.common.enums.userPush.SpecialPrivilegeEnum;
import com.qmth.paper.library.common.service.*;
import com.qmth.paper.library.common.util.Result;
import com.qmth.paper.library.common.util.ResultUtil;
import com.qmth.paper.library.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.File;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Date: 2021/3/30.
 */
@Api(tags = "系统Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.common}")
public class SysController {
    private final static Logger log = LoggerFactory.getLogger(SysController.class);

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicVerifyCodeService basicVerifyCodeService;

    @Resource
    DictionaryConfig dictionaryConfig;

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    PaperLibraryCommonService paperLibraryCommonService;

    @Resource
    TBTaskService tbTaskService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    SysUserRoleService sysUserRoleService;

    @Resource
    LibraryCommonService libraryCommonService;

    @Resource
    BasicStudentService basicStudentService;

    @Resource
    BasicSemesterService basicSemesterService;

    @Resource
    AuthInfoService authInfoService;

    /**
     * 登录
     *
     * @param login
     * @return
     */
    @ApiOperation(value = "登录")
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "用户信息", response = LoginResult.class)})
    @Aac(auth = BOOL.FALSE)
    public Result login(@ApiParam(value = "用户信息", required = true) @Valid @RequestBody LoginParam login, BindingResult bindingResult) throws NoSuchAlgorithmException {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        if (Objects.nonNull(login.getSchoolCode()) && login.getSchoolCode().contains("#")) {
            login.setSchoolCode(login.getSchoolCode().substring(0, login.getSchoolCode().indexOf("#")));
        }

        BasicSchool basicSchool = null;
        if (!login.getSchoolCode().equalsIgnoreCase(SystemConstant.ADMIN_CODE)) {
            basicSchool = commonCacheService.schoolCache(login.getSchoolCode());
            if (Objects.isNull(basicSchool)) {
                throw ExceptionResultEnum.SCHOOL_NO_DATA.exception();
            }
            if (Objects.nonNull(basicSchool.getEnable()) && !basicSchool.getEnable()) {
                throw ExceptionResultEnum.SCHOOL_ENABLE.exception();
            }
        }
        QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
        if (Objects.nonNull(basicSchool)) {
            wrapper.lambda().eq(SysUser::getSchoolId, basicSchool.getId());
        }
        // 账号密码登录
        List<SysUser> userList = null;
        if (LoginTypeEnum.ACCOUNT.name().equals(login.getType())) {
            String loginName = login.getLoginName();
            String password = login.getPassword();
            // 非空校验
            if (StringUtils.isBlank(loginName)) {
                throw ExceptionResultEnum.ERROR.exception("用户名不能为空");
            }
            if (StringUtils.isBlank(password)) {
                throw ExceptionResultEnum.ERROR.exception("密码不能为空");
            }
            wrapper.lambda().and(w -> w.eq(SysUser::getLoginName, loginName).or().eq(SysUser::getCode, loginName));
            userList = sysUserService.list(wrapper);

            //用户不存在
            if (userList == null || userList.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("用户不存在");
            }
            if (userList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("查出多个用户");
            }

            SysUser sysUser = userList.get(0);
            if (!sysUser.getEnable()) {
                throw ExceptionResultEnum.ERROR.exception("用户被禁用");
            }

            if (!password.equals(userList.get(0).getPassword())) {
                throw ExceptionResultEnum.ERROR.exception("密码错误");
            }
        } else if (LoginTypeEnum.PHONE.name().equals(login.getType())) {
            String mobileNumber = login.getMobileNumber();
            String code = login.getCode();
            // 非空校验
            if (StringUtils.isBlank(mobileNumber)) {
                throw ExceptionResultEnum.ERROR.exception("手机号不能为空");
            }
            if (StringUtils.isBlank(code)) {
                throw ExceptionResultEnum.ERROR.exception("验证码不能为空");
            }
            wrapper.lambda().eq(SysUser::getMobileNumber, mobileNumber);
            userList = sysUserService.list(wrapper);

            //用户不存在
            if (userList == null || userList.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("用户不存在");
            }
            if (userList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("查出多个用户");
            }
            SysUser sysUser = userList.get(0);
            // 校验验证码
            sysUserService.checkSmsCode(sysUser.getId(), sysUser.getMobileNumber(), code);

            // 如果不是共用验证码再过期
            if (!dictionaryConfig.smsDomain().getSmsNormalCode().equals(login.getCode())) {
                sysUserService.expiredVerifyCode(sysUser.getId(), sysUser.getMobileNumber());
            }
        } else {
            throw ExceptionResultEnum.ERROR.exception("登录参数错误");
        }

        if (login.getSchoolCode().equalsIgnoreCase(SystemConstant.ADMIN_CODE)) {
            userList.forEach(o -> {
                AuthBean authBean = libraryCommonService.getUserAuth(o.getId());
                if (Objects.nonNull(authBean) && Objects.nonNull(authBean.getRoleList()) && authBean.getRoleList().size() > 0) {
                    if (Objects.nonNull(authBean.getSchool())) {
                        throw ExceptionResultEnum.ERROR.exception("用户无法通过当前页面登录");
                    }
                }
            });
        }
        SysUser sysUser = userList.get(0);
        LoginResult loginResult = libraryCommonService.login(login.getPassword(), sysUser, AppSourceEnum.SYSTEM);

        return ResultUtil.ok(loginResult);
    }

    /**
     * 登出
     *
     * @return
     */
    @ApiOperation(value = "登出")
    @RequestMapping(value = "/logout", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result logout() throws NoSuchAlgorithmException {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        libraryCommonService.removeUserInfo(sysUser.getId(), false);
        return ResultUtil.ok();
    }

    /**
     * 发送验证码
     *
     * @param loginParam
     * @return
     */
    @ApiOperation(value = "发送验证码")
    @RequestMapping(value = "/get_verify_code", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result getVerifyCode(@RequestBody LoginParam loginParam) {
        String mobileNumber = loginParam.getMobileNumber();
        if (StringUtils.isBlank(mobileNumber)) {
            throw ExceptionResultEnum.ERROR.exception("手机号不能为空");
        }
        QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(loginParam.getSchoolCode())) {
            BasicSchool basicSchool = commonCacheService.schoolCache(loginParam.getSchoolCode());
            if (Objects.nonNull(basicSchool)) {
                wrapper.lambda().eq(SysUser::getSchoolId, basicSchool.getId());
            }
        }
        wrapper.lambda().eq(SysUser::getMobileNumber, mobileNumber);
        List<SysUser> userList = sysUserService.list(wrapper);

        //用户不存在
        if (userList == null || userList.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("手机号不存在");
        }
        if (userList.size() > 1) {
            throw ExceptionResultEnum.ERROR.exception("手机号绑定了多个用户");
        }
        SysUser sysUser = userList.get(0);

        if (!sysUser.getEnable()) {
            throw ExceptionResultEnum.ERROR.exception("手机号被禁用");
        }

        if (SystemConstant.strNotNull(mobileNumber)) {
            basicVerifyCodeService.sendVeirfyCode(mobileNumber, sysUser);
        }
        int pwdCount = sysUser.getPwdCount();
        List<SysRole> sysRoleList = sysUserRoleService.listRoleByUserId(sysUser.getId());
        if (sysRoleList.stream().map(SysRole::getType).collect(Collectors.toSet()).contains(RoleTypeEnum.ADMIN)) {
            mobileNumber = sysUser.getLoginName() + "（特权用户）";
            pwdCount = 1;
        }
        UserLoginCheckResult userLoginCheckResult = new UserLoginCheckResult(sysUser.getId(), mobileNumber, pwdCount);
        return ResultUtil.ok(userLoginCheckResult, "");
    }


    /**
     * 发送验证码（绑定手机号）
     *
     * @param loginParam
     * @return
     */
    @ApiOperation(value = "发送验证码（绑定手机号）")
    @RequestMapping(value = "/get_verify_code_for_bind", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result getVerifyCodeForBind(@RequestBody LoginParam loginParam) {
        String loginName = loginParam.getLoginName();
        String password = loginParam.getPassword();
        String schoolCode = loginParam.getSchoolCode();
        String mobileNumber = loginParam.getMobileNumber();

        if (StringUtils.isBlank(schoolCode)) {
            throw ExceptionResultEnum.ERROR.exception("学校代码不能为空");
        }
        QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
        BasicSchool basicSchool = commonCacheService.schoolCache(loginParam.getSchoolCode());
        if (Objects.isNull(basicSchool)) {
            throw ExceptionResultEnum.ERROR.exception("学校不存在");
        }

        if (StringUtils.isBlank(mobileNumber)) {
            throw ExceptionResultEnum.ERROR.exception("手机号不能为空");
        }

        wrapper.lambda().eq(SysUser::getSchoolId, basicSchool.getId())
                .eq(SysUser::getMobileNumber, mobileNumber);
        List<SysUser> userList = sysUserService.list(wrapper);

        if (userList != null && userList.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("手机号已使用");
        }

        // 根据账号/密码取用户ID
        if (StringUtils.isBlank(loginName)) {
            throw ExceptionResultEnum.ERROR.exception("账号不能为空");
        }
        if (StringUtils.isBlank(password)) {
            throw ExceptionResultEnum.ERROR.exception("密码不能为空");
        }
        QueryWrapper<SysUser> sysUserQueryWrapper = new QueryWrapper<>();
        sysUserQueryWrapper.lambda().eq(SysUser::getSchoolId, basicSchool.getId())
                .and(w -> w.eq(SysUser::getLoginName, loginName).or().eq(SysUser::getCode, loginName))
                .eq(SysUser::getPassword, password);
        SysUser sysUser = sysUserService.getOne(sysUserQueryWrapper);
        if (sysUser == null) {
            throw ExceptionResultEnum.ERROR.exception("用户数据异常");
        }

        // 发送短信验证码
        basicVerifyCodeService.sendVeirfyCode(mobileNumber, sysUser);

        int pwdCount = sysUser.getPwdCount();
        List<SysRole> sysRoleList = sysUserRoleService.listRoleByUserId(sysUser.getId());
        if (sysRoleList.stream().map(SysRole::getType).collect(Collectors.toSet()).contains(RoleTypeEnum.ADMIN)) {
            mobileNumber = sysUser.getLoginName() + "（特权用户）";
            pwdCount = 1;
        }
        UserLoginCheckResult userLoginCheckResult = new UserLoginCheckResult(sysUser.getId(), mobileNumber, pwdCount);
        return ResultUtil.ok(userLoginCheckResult, "");
    }

    @ApiOperation(value = "根据机构代码查询机构信息接口")
    @RequestMapping(value = "/school/query_by_school_code", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "学校信息", response = EditResult.class)})
    @Aac(auth = BOOL.FALSE)
    public Result queryBySchoolCode(@ApiParam(value = "机构code", required = true) @RequestParam String code) {
        if (Objects.nonNull(code) && code.contains("#")) {
            code = code.substring(0, code.indexOf("#"));
        }
        if (!code.equalsIgnoreCase(SystemConstant.ADMIN_CODE)) {
            BasicSchool basicSchool = commonCacheService.schoolCache(code);
            Optional.ofNullable(basicSchool).orElseThrow(() -> ExceptionResultEnum.SCHOOL_NO_DATA.exception());
            authInfoService.appHasExpired(code);
            Map<String, String> map = new HashMap<>();
            if (Objects.nonNull(basicSchool.getLogo()) && (!basicSchool.getLogo().startsWith("http") || !basicSchool.getLogo().startsWith("https"))) {
                String hostUrl = SystemConstant.getLocalFileHost(dictionaryConfig.fssLocalFileDomain().getServer());
                String filePath = SystemConstant.getLocalFilePath(hostUrl, basicSchool.getLogo());
                String url = null;
                if (filePath.startsWith("/")) {
                    url = hostUrl + filePath;
                } else {
                    url = hostUrl + File.separator + filePath;
                }
                map.put(SystemConstant.LOGO, url);
            } else {
                map.put(SystemConstant.LOGO, basicSchool.getLogo());
            }
            map.put("name", basicSchool.getName());
            return ResultUtil.ok(map);
        } else {
            Map<String, String> map = new HashMap<>();
            map.put(SystemConstant.LOGO, dictionaryConfig.sysDomain().getAdminLogoUrl());
            map.put("name", null);
            return ResultUtil.ok(map);
        }
    }

    @ApiOperation(value = "文件上传接口")
    @RequestMapping(value = "/file/upload", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result fileUpload(@ApiParam(value = "上传文件", required = true) @RequestParam MultipartFile file,
                             @ApiParam(value = "上传文件类型", required = true) @RequestParam UploadFileEnum type) {
        BasicAttachment basicAttachment = null;
        try {
            basicAttachment = paperLibraryCommonService.saveAttachment(file, ServletUtil.getRequestMd5(), type);
            if (Objects.isNull(basicAttachment)) {
                throw ExceptionResultEnum.ATTACHMENT_ERROR.exception();
            }
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
            if (Objects.nonNull(basicAttachment)) {
                basicAttachmentService.deleteAttachment(basicAttachment);
            }
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return ResultUtil.ok(new EditResult(basicAttachment.getId(), libraryCommonService.filePreview(basicAttachment.getPath()), basicAttachment.getPages()));
    }


    @ApiOperation(value = "文件预览接口")
    @RequestMapping(value = "/file/preview", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result filePreview(@ApiParam(value = "附件id", required = false, defaultValue = "") @RequestParam(value = "id", required = false, defaultValue = "") String id) {
        if (StringUtils.isBlank(id)) {
            throw ExceptionResultEnum.ERROR.exception("没有附件");
        } else {
            BasicAttachment basicAttachment = basicAttachmentService.getById(Long.parseLong(id));
            return ResultUtil.ok(new EditResult(libraryCommonService.filePreview(basicAttachment.getPath())));
        }
    }

    @ApiOperation(value = "查询用户权限")
    @RequestMapping(value = "/get_menu", method = RequestMethod.POST)
    public Result getMenu() {
        return ResultUtil.ok(sysUserRoleService.listByUserId());
    }

    @ApiOperation(value = "获取服务器时间")
    @RequestMapping(value = "/get_system_time", method = RequestMethod.POST)
    @Aac(auth = BOOL.FALSE)
    public Result getSystemTime() {
        return ResultUtil.ok(System.currentTimeMillis());
    }

    @ApiOperation(value = "获取系统编码")
    @RequestMapping(value = "/get_code", method = RequestMethod.POST)
    public Result getCode(@ApiParam(value = "编码类型", required = true) @RequestParam SystemCodeEnum type) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        String number = libraryCommonService.getSysIncrCode(type, sysUser);
        return ResultUtil.ok((Object) number);
    }

    @ApiOperation(value = "共用接口-查询字典")
    @RequestMapping(value = "/get_dictionary", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result findDictionaryList(@ApiParam(value = "学生学院id") @RequestParam(required = false) String collegeId,
                                     @ApiParam(value = "专业id") @RequestParam(required = false) String majorName,
                                     @ApiParam(value = "班级id") @RequestParam(required = false) String clazzName,
                                     @ApiParam(value = "学生id") @RequestParam(required = false) String studentId,
                                     @ApiParam(value = "查询字典对象", required = true) @RequestParam DictionaryEnum dictionaryEnum) {
        Long schoolId = SystemConstant.convertIdToLong(ServletUtil.getRequestHeaderSchoolId().toString());
        List<DictionaryResult> dictionaryResultList = new ArrayList<>();
        switch (dictionaryEnum) {
            case SEMESTER:
                List<BasicSemester> basicSemesterList = basicSemesterService.list(new QueryWrapper<BasicSemester>()
                        .lambda()
                        .eq(BasicSemester::getSchoolId, schoolId)
                        .eq(BasicSemester::getEnable, true)
                        .orderByDesc(BasicSemester::getInUsed));
                dictionaryResultList = basicSemesterList.stream().map(e -> {
                    DictionaryResult dictionaryResult = new DictionaryResult();
                    dictionaryResult.setId(e.getId());
                    dictionaryResult.setCode(e.getCode());
                    dictionaryResult.setName(e.getName());
                    dictionaryResult.setInUsed(e.getInUsed());
                    return dictionaryResult;
                }).collect(Collectors.toList());
                break;
            case STUDENT:
                QueryWrapper<BasicStudent> studentQueryWrapper = new QueryWrapper<>();
                studentQueryWrapper.lambda().eq(BasicStudent::getSchoolId, schoolId).eq(BasicStudent::getEnable, true);
                if (SystemConstant.longNotNull(SystemConstant.convertIdToLong(collegeId))) {
                    studentQueryWrapper.lambda().eq(BasicStudent::getBelongOrgId, collegeId);
                }
                if (SystemConstant.strNotNull(majorName)) {
                    studentQueryWrapper.lambda().eq(BasicStudent::getMajorName, majorName);
                }
                if (SystemConstant.strNotNull(clazzName)) {
                    studentQueryWrapper.lambda().eq(BasicStudent::getClazzName, clazzName);
                }
                if (SystemConstant.longNotNull(SystemConstant.convertIdToLong(studentId))) {
                    studentQueryWrapper.lambda().eq(BasicStudent::getId, studentId);
                }
                List<BasicStudent> basicStudentList = basicStudentService.list(studentQueryWrapper);
                dictionaryResultList = basicStudentList.stream().map(e -> {
                    DictionaryResult dictionaryResult = new DictionaryResult();
                    dictionaryResult.setId(e.getId());
                    dictionaryResult.setCode(e.getStudentCode());
                    dictionaryResult.setName(e.getStudentName());
                    return dictionaryResult;
                }).collect(Collectors.toList());
                break;
            default:
                break;
        }
        return ResultUtil.ok(dictionaryResultList);
    }
}
